package com.persagy.apm.energyalarmstarter.alarmdata.aspect;

import com.persagy.apm.energyalarmstarter.alarmdata.feign.AlarmUrlParam;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.UrlParamDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.util.Arrays;
import java.util.List;

/**
 * AlarmClient切面，处理feign调用接口，接口返回值异常的情况
 *
 * @author lixing
 * @version V1.0 2021/9/15 8:18 下午
 */
@Slf4j
@Aspect
@Component
public class AlarmClientAspect {
    @Pointcut("execution(public * com.persagy.apm.energyalarmstarter.alarmdata.feign.client.*.*(..))")
    public void feignPointCut() {
    }

    @Before("feignPointCut()")
    public void before(JoinPoint joinPoint) {
        List<Object> args = Arrays.asList(joinPoint.getArgs());
        if (CollectionUtils.isEmpty(args)) {
            return;
        }
        for (Object arg : args) {
            if (!(arg instanceof AlarmUrlParam)) {
                continue;
            }
            // 如果未传某些参数，设置为默认值
            AlarmUrlParam argParam = (AlarmUrlParam) arg;
            if (StringUtils.isBlank(argParam.getGroupCode())) {
                argParam.setGroupCode("0");
            }
            if (StringUtils.isBlank(argParam.getProjectId())) {
                argParam.setProjectId("0");
            }
            if (StringUtils.isBlank(argParam.getUserId())) {
                argParam.setUserId("System");
            }
            if (StringUtils.isBlank(argParam.getAppId())) {
                argParam.setAppId("0");
            }
        }
    }

    @AfterReturning(returning = "res", pointcut = "feignPointCut()")
    public void after(DmpResult res) {
        if (DmpResult.FAILURE.equals(res.getResult())) {
            throw new RuntimeException(res.getMessage());
        }
    }
}
