package com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.persagy.apm.energyalarmstarter.alarmdata.enums.EnumAlarmListType;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.UrlParamDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DateUtils;
import lombok.AccessLevel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 报警记录
 *
 * @author lixing
 * @version V1.0 2021/9/16 9:33 上午
 **/
@EqualsAndHashCode(callSuper = true)
@Data
@JsonInclude(JsonInclude.Include.NON_NULL)
public class QueryAlarmRecordDTO extends UrlParamDTO {
    /**
     * 报警记录id
     */
    private List<String> id;
    /**
     * 报警条目编码
     */
    private List<String> itemCode;
    /**
     * 报警对象id
     */
    private List<String> objId;
    /**
     * 设备类编码
     */
    private List<String> classCode;
    /**
     * 报警等级
     */
    private List<String> level;
    /**
     * 备注
     */
    private String remark;
    /**
     * 报警名称
     */
    private String name;
    /**
     * 报警系统
     */
    private List<String> category;
    /**
     * 报警状态
     */
    private List<Integer> state;
    /**
     * 报警性质
     */
    private List<Integer> nature;
    /**
     * 处理方式
     */
    private List<Integer> treatMode;
    /**
     * 处理状态
     */
    private List<Integer> treatState;
    /**
     * 报警发生时间
     */
    private Date triggerTime;
    /**
     * 报警发生时间区间-起始时间（禁用get方法）
     */
    @Getter(AccessLevel.NONE)
    private Date triggerDurationStart;
    /**
     * 报警发生时间区间-结束时间（禁用get方法）
     */
    @Getter(AccessLevel.NONE)
    private Date triggerDurationEnd;
    /**
     * 工单id
     */
    private List<String> orderId;
    /**
     * 工单状态
     */
    private List<String> orderState;
    /**
     * 项目id
     */
    @Getter(AccessLevel.NONE)
    private List<String> projectIds;

    /**
     * 列表类型（当前列表、历史列表）
     */
    private EnumAlarmListType listType;

    public List<String> getProjectId() {
        // 使用projectIds字段来存储查询条件中包含的项目id，和UrlParamDTO中的projectId做一个区分
        if (!CollectionUtils.isEmpty(projectIds)) {
            return projectIds;
        }
        return null;
    }

    public JSONObject getName() {
        // 名称模糊查询
        if (this.name == null) {
            return null;
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("$like", this.name);
        return jsonObject;
    }

    public Object getTriggerTime() {
        // 触发时间可以精确查询也可以根据区间查询
        if (this.triggerTime != null) {
            return DateUtils.date2Str(this.triggerTime, DateUtils.SDFSECOND);
        }

        if (this.triggerDurationStart !=null || this.triggerDurationEnd != null) {
            JSONObject jsonObject = new JSONObject();
            if (this.triggerDurationStart !=null) {
                jsonObject.put("$gte", DateUtils.date2Str(this.triggerDurationStart, DateUtils.SDFSECOND));
            }
            if (this.triggerDurationEnd !=null) {
                jsonObject.put("$lte", DateUtils.date2Str(this.triggerDurationEnd, DateUtils.SDFSECOND));
            }
            return jsonObject;
        }
        return null;
    }
}
