package com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.AlarmUrlParam;
import lombok.*;

import java.io.Serializable;

/**
 * @description: 报警请求 url 通用参数
 * @author: xingmaojun
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/10/28 11:50
 * @version: V1.0
 **/
@Builder
@Setter
@NoArgsConstructor
@AllArgsConstructor
@JsonInclude(JsonInclude.Include.NON_NULL)
public class UrlParamDTO implements Serializable {
    /**
     * 用户id
     */
    private String userId;

    /**
     * 集团编码
     */
    private String groupCode;

    /**
     * 项目Id
     */
    private String projectId;

    /**
     * 应用id
     */
    private String appId;

    /**
     * 获取应该放到url中的参数
     *
     * @return 应该放到url中的参数
     * @author lixing
     * @version V1.0 2021/9/16 3:11 下午
     */
    public AlarmUrlParam fetchParams() {
        return new AlarmUrlParam(this.userId, this.groupCode, this.projectId, this.appId);
    }

    /**
     * 初始化应该放到url中的参数
     *
     * @param fetchParams 应该放到url中的参数
     * @author lixing
     * @version V1.0 2021/9/23 9:57 上午
     */
    public void initUrlParams(AlarmUrlParam fetchParams) {
        if (fetchParams == null) {
            return;
        }
        this.userId = fetchParams.getUserId();
        this.groupCode = fetchParams.getGroupCode();
        this.projectId = fetchParams.getProjectId();
        this.appId = fetchParams.getAppId();
    }

    /**
     * url中的参数设置为默认值
     *
     * @author lixing
     * @version V1.0 2021/9/23 9:57 上午
     */
    public void setDefaultUrlParam() {
        this.userId = "system";
        this.groupCode = "0";
        this.projectId = "0";
        this.appId = "0";
    }
}
