package com.persagy.apm.energyalarmstarter.alarmdata.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.client.AlarmClient;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmcomment.AddAlarmCommentDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmcomment.QueryAlarmCommentDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.OrderDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.QueryDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmCommentItem;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.DmpUpsertVO;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DmpResultUtil;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.Obj2JSONObject;

/**
 * 报警批注service类
 *
 * @author lixing
 * @version V1.0 2021/9/15 8:47 下午
 **/
@Service
public class AlarmCommentServiceImpl {

    @Autowired
    AlarmClient alarmClient;

    /**
     * 查询报警批注
     *
     * @param queryAlarmCommentDTO 入参
     * @return 报警批注列表
     * @throws Exception
     */
    public DmpResult<List<AlarmCommentItem>> query(QueryAlarmCommentDTO queryAlarmCommentDTO) throws Exception {
        QueryDTO<QueryAlarmCommentDTO> queryDTO = new QueryDTO<>();
        queryDTO.initUrlParams(queryAlarmCommentDTO.fetchParams());
        queryDTO.setCriteria(queryAlarmCommentDTO);
        return pageQuery(queryDTO);
    }

    /**
     * 分页查询报警批注
     *
     * @param queryDTO 入参
     * @return 报警批注列表
     * @throws Exception
     */
    public DmpResult<List<AlarmCommentItem>> pageQuery(QueryDTO<QueryAlarmCommentDTO> queryDTO) throws Exception {
        // 按创建时间倒序排列
        OrderDTO orderDTO = new OrderDTO();
        orderDTO.setColumn("create_time");
        orderDTO.setAsc(false);
        List<OrderDTO> orders = Lists.newArrayList(orderDTO);
        queryDTO.setOrders(orders);

        JSONObject jsonObject = Obj2JSONObject.convert(queryDTO);
        DmpResult<JSONArray> result = alarmClient.queryAlarmComment(queryDTO.fetchParams(), jsonObject);
        return DmpResultUtil.multiConvert(result, AlarmCommentItem.class);
    }

    /**
     * 创建报警批注
     *
     * @param addAlarmCommentDTO 入参
     * @return 报警批注id
     * @throws Exception
     */
    public DmpResult<DmpUpsertVO> create(AddAlarmCommentDTO addAlarmCommentDTO) throws Exception {
        JSONObject jsonObject = Obj2JSONObject.convert(addAlarmCommentDTO);
        DmpResult<JSONObject> alarmComment = alarmClient.createAlarmComment(addAlarmCommentDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(alarmComment, DmpUpsertVO.class);
    }

}
