package com.persagy.apm.energyalarmstarter.alarmdata.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.client.AlarmClient;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.*;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.QueryAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.QueryDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmConfigItem;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.DmpBatchUpsertVO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.DmpUpsertVO;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DmpResultUtil;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.Obj2JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 报警条件service类
 *
 * @author lixing
 * @version V1.0 2021/9/15 8:47 下午
 **/
@Service
public class AlarmConfigServiceImpl {
    @Autowired
    AlarmClient alarmClient;

    /**
     * 创建报警条件
     *
     * @param addAlarmConfigDTO 入参
     * @return 报警条件id
     * @author lixing
     * @version V1.0 2021/9/15 9:22 下午
     */
    public DmpResult<DmpUpsertVO> create(AddAlarmConfigDTO addAlarmConfigDTO) throws Exception{
        JSONObject jsonObject = Obj2JSONObject.convert(addAlarmConfigDTO);
        DmpResult<JSONObject> alarmConfig = alarmClient.createAlarmConfig(addAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(alarmConfig, DmpUpsertVO.class);
    }

    /**
     * 查询报警条件
     *
     * @param queryAlarmConfigDTO 入参
     * @return 报警条件列表（不分页）
     * @author lixing
     * @version V1.0 2021/9/16 9:51 上午
     */
    public DmpResult<List<AlarmConfigItem>> query(QueryAlarmConfigDTO queryAlarmConfigDTO) throws Exception {
        QueryDTO<QueryAlarmConfigDTO> queryDTO = new QueryDTO<>();
        queryDTO.setCriteria(queryAlarmConfigDTO);
        JSONObject jsonObject = Obj2JSONObject.convert(queryDTO);
        DmpResult<JSONArray> result = alarmClient.queryAlarmConfig(queryAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.multiConvert(result, AlarmConfigItem.class);
    }

    /**
     * 分页查询报警条件
     *
     * @param queryAlarmConfigDTO 入参
     * @return 报警条件列表（分页）
     * @author lixing
     * @version V1.0 2021/9/16 9:51 上午
     */
    public DmpResult<List<AlarmConfigItem>> query(QueryDTO<QueryAlarmConfigDTO> queryAlarmConfigDTO) throws Exception {
        JSONObject jsonObject = Obj2JSONObject.convert(queryAlarmConfigDTO);
        DmpResult<JSONArray> result = alarmClient.queryAlarmConfig(queryAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.multiConvert(result, AlarmConfigItem.class);
    }

    /**
     * 更新报警条件
     *
     * @param updateAlarmConfigDTO 入参
     * @return 报警条件id
     * @author lixing
     * @version V1.0 2021/9/15 9:22 下午
     */
    public DmpResult<DmpUpsertVO> update(UpdateAlarmConfigDTO updateAlarmConfigDTO) throws Exception{
        JSONObject jsonObject = Obj2JSONObject.convert(updateAlarmConfigDTO);
        DmpResult<JSONObject> alarmConfig = alarmClient.updateAlarmConfig(updateAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(alarmConfig, DmpUpsertVO.class);
    }

    /**
     * 批量创建
     *
     * @param batchAddAlarmConfigDTO 入参
     * @return 报警条件ids
     * @author lixing
     * @version V1.0 2021/9/16 9:37 下午
     */
    public DmpResult<DmpBatchUpsertVO> batchCreate(BatchAddAlarmConfigDTO batchAddAlarmConfigDTO) throws Exception {
        JSONObject jsonObject = Obj2JSONObject.convert(batchAddAlarmConfigDTO);
        DmpResult<JSONObject> result = alarmClient.batchCreateAlarmConfig(batchAddAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(result, DmpBatchUpsertVO.class);
    }

    /**
     * 批量更新
     *
     * @param batchUpdateAlarmConfigDTO 入参
     * @return 报警条件ids
     * @author lixing
     * @version V1.0 2021/9/16 9:38 下午
     */
    public DmpResult<DmpBatchUpsertVO> batchUpdate(BatchUpdateAlarmConfigDTO batchUpdateAlarmConfigDTO) throws Exception {
        JSONObject jsonObject = Obj2JSONObject.convert(batchUpdateAlarmConfigDTO);
        DmpResult<JSONObject> result = alarmClient.batchUpdateAlarmConfig(batchUpdateAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(result, DmpBatchUpsertVO.class);
    }

    /**
     * 删除
     *
     * @param deleteAlarmConfigDTO 入参
     * @return nothing
     * @author lixing
     * @version V1.0 2021/9/16 9:38 下午
     */
    public DmpResult<DmpBatchUpsertVO> delete(DeleteAlarmConfigDTO deleteAlarmConfigDTO) throws Exception {
        JSONObject jsonObject = Obj2JSONObject.convert(deleteAlarmConfigDTO);
        DmpResult<JSONObject> result = alarmClient.deleteAlarmConfig(deleteAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(result, DmpBatchUpsertVO.class);
    }

    /**
     * 批量删除
     *
     * @param batchDeleteAlarmConfigDTO 入参
     * @return nothing
     * @author lixing
     * @version V1.0 2021/9/16 9:38 下午
     */
    public DmpResult<DmpBatchUpsertVO> batchDelete(BatchDeleteAlarmConfigDTO batchDeleteAlarmConfigDTO) throws Exception {
        JSONObject jsonObject = Obj2JSONObject.convert(batchDeleteAlarmConfigDTO);
        DmpResult<JSONObject> result = alarmClient.batchDeleteAlarmConfig(batchDeleteAlarmConfigDTO.fetchParams(), jsonObject);
        return DmpResultUtil.singleConvert(result, DmpBatchUpsertVO.class);
    }

//    public static void main(String[] args) throws Exception{
//        AddAlarmConfigDTO addAlarmConfigDTO = new AddAlarmConfigDTO();
//        addAlarmConfigDTO.setObjId("1");
//        AlarmCondition alarmCondition = new AlarmCondition();
//        alarmCondition.setConfigs("configs");
//        alarmCondition.setTrigger("trigger");
//        addAlarmConfigDTO.setCondition(alarmCondition);
//        JSONObject jsonObject = Obj2JSONObject.convert(addAlarmConfigDTO);
//        System.out.println(jsonObject);
//    }
}
