package com.persagy.apm.energyalarmstarter.alarmdata.utils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 数据中台返回结果处理类
 *
 * @author lixing
 * @version V1.0 2021/9/15 11:29 下午
 **/
public class DmpResultUtil {
    /**
     * 单个对象转换
     *
     * @param result 原始结果
     * @param clazz 需要转换的对象类
     * @return DmpResult(转换后的对象)
     * @author lixing
     * @version V1.0 2021/9/15 11:46 下午
     */
    public static <T> DmpResult<T> singleConvert(DmpResult<JSONObject> result, Class<T> clazz) throws Exception {
        if (result == null) {
            return null;
        }
        JSONObject data = result.getData();
        T reverse = Obj2JSONObject.reverse(data, clazz);

        DmpResult<T> tmp = copyBasicInfo(result);
        tmp.setData(reverse);
        return tmp;
    }

    /**
     * 单个对象转换
     *
     * @param result 原始结果
     * @param clazz 需要转换的对象类
     * @return DmpResult(转换后的对象)
     * @author lixing
     * @version V1.0 2021/9/15 11:46 下午
     */
    public static <T> DmpResult<List<T>> multiConvert(DmpResult<JSONArray> result, Class<T> clazz) throws Exception {
        if (result == null) {
            return null;
        }
        DmpResult<List<T>> newResult = copyBasicInfo(result);
        List<T> resultData = new ArrayList<>();
        newResult.setData(resultData);

        JSONArray data = result.getData();
        if (data == null) {
            return newResult;
        }
        for (int i = 0; i < data.size(); i++) {
            JSONObject dataObj = data.getJSONObject(i);
            T reverse = Obj2JSONObject.reverse(dataObj, clazz);
            resultData.add(reverse);
        }
        return newResult;
    }

    /**
     * 拷贝基础信息
     *
     * @param result 数据中台结果
     * @return 只要基础信息，不要返回的数据
     * @author lixing
     * @version V1.0 2021/11/18 8:47 下午
     */
    private static <T> DmpResult<T> copyBasicInfo(DmpResult result) {
        DmpResult<T> tmp = new DmpResult<>();
        tmp.setResult(result.getResult());
        tmp.setCount(result.getCount());
        tmp.setMessage(result.getMessage());
        return tmp;
    }

    /**
     * 获取数据中台结果中的数据
     *
     * @param dmpResult 数据中台结果
     * @return 结果中的数据，不要基础信息
     * @author lixing
     * @version V1.0 2021/11/18 8:48 下午
     */
    public static <T>  T getData(DmpResult<T> dmpResult) {
        String result = dmpResult.getResult();
        if(StringUtils.isBlank(result)||!result.equals(DmpResult.SUCCESS)){
            return  null;
        }
        return dmpResult.getData();
    }

}
