package com.persagy.apm.energyalarmstarter.alarmengine.feign.client;

import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.*;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.fallback.EnergyAlarmServiceClientFallbackFactory;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

/**
 * @description: feign调用数据中台报警接口
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/27 10:43 上午
 * @version: V1.0
 */
@FeignClient(name = "energy-alarm-service", fallbackFactory = EnergyAlarmServiceClientFallbackFactory.class)
public interface EnergyAlarmServiceClient {
    /**
     * 全量查询报警条件
     *
     * @param baseRequestBody 查询条件
     * @return 报警条件列表
     * @author lixing
     * @version V1.0 2021/10/22 10:33 上午
     */
    @PostMapping("alarmConditions/query_all")
    CommonResult<List<AlarmCondition>> queryAllAlarmCondition(@RequestBody BaseRequestBody baseRequestBody);

    /**
     * 全量查询项目
     *
     * @param baseRequestBody 查询条件
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/10/22 2:33 下午
     */
    @PostMapping("projectAlarmRules/projects")
    CommonResult<List<ProjectVO>> queryProjects(@RequestBody BaseRequestBody baseRequestBody);

    /**
     * 全量查询项目上设备与报警条件的关联
     *
     * @param baseRequestBody 查询条件
     * @return 设备与报警条件的关联
     * @author lixing
     * @version V1.0 2021/10/22 2:33 下午
     */
    @PostMapping("/projectAlarmRuleObjRels/query_obj_condition_rels")
    CommonResult<List<ObjConditionRel>> queryObjAlarmConditionRel(@RequestBody BaseRequestBody baseRequestBody);
}
