package com.persagy.apm.energyalarmstarter.alarmengine.feign.service;

import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.AlarmCondition;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.BaseRequestBody;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.ObjConditionRel;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.ProjectVO;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.client.EnergyAlarmServiceClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 报警业务层service类
 *
 * @author lixing
 * @version V1.0 2021/10/22 2:36 下午
 **/
@Service
public class AlarmServiceImpl {
    @Autowired
    EnergyAlarmServiceClient energyAlarmServiceClient;

    /** 设置一个假的用户密码 */
    private final String pd = "pd";

    /**
     * 全量查询报警条件
     *
     * @param userId    用户id
     * @param groupCode 集团编码
     * @return 报警条件列表
     * @author lixing
     * @version V1.0 2021/10/22 10:33 上午
     */
    public List<AlarmCondition> queryAllAlarmCondition(String userId, String groupCode) {
        CommonResult<List<AlarmCondition>> listCommonResult = energyAlarmServiceClient.queryAllAlarmCondition(
                BaseRequestBody.builder().userId(userId).groupCode(groupCode).pd(pd).build());
        if (listCommonResult == null) {
            return new ArrayList<>();
        }
        return listCommonResult.getContent();
    }

    /**
     * 全量查询项目
     *
     * @param userId    用户id
     * @param groupCode 集团编码
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/10/22 2:33 下午
     */
    public List<ProjectVO> queryProjects(String userId, String groupCode) {
        CommonResult<List<ProjectVO>> listCommonResult = energyAlarmServiceClient.queryProjects(
                BaseRequestBody.builder().userId(userId).groupCode(groupCode).pd(pd).build());
        if (listCommonResult == null) {
            return new ArrayList<>();
        }
        return listCommonResult.getContent();
    }

    /**
     * 全量查询项目上设备与报警条件的关联
     *
     * @param userId    用户id
     * @param groupCode 集团编码
     * @param projectId 项目id
     * @return 设备与报警条件的关联
     * @author lixing
     * @version V1.0 2021/10/22 2:33 下午
     */
    public List<ObjConditionRel> queryObjAlarmConditionRel(String userId, String groupCode, String projectId) {
        CommonResult<List<ObjConditionRel>> listCommonResult = energyAlarmServiceClient.queryObjAlarmConditionRel(
                BaseRequestBody.builder().userId(userId).groupCode(groupCode).pd(pd).projectId(projectId).build());
        if (listCommonResult == null) {
            return new ArrayList<>();
        }
        return listCommonResult.getContent();
    }
}
