package com.persagy.apm.energyalarmstarter.alarmengine.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.AlarmUrlParam;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmitem.QueryAlarmItemDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.client.AlarmClient;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmItem;
import com.persagy.apm.energyalarmstarter.alarmdata.mq.model.DmpMessage;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmItemServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DmpResultUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 处理报警引擎发送的报警创建、更新、持续消息
 *
 * @author lixing
 * @version V1.0 2021/11/16 8:24 下午
 */
@Slf4j
public abstract class AlarmRecordMsgHandler extends BaseService {
    @Autowired
    AlarmClient alarmClient;
    @Autowired
    AlarmItemServiceImpl alarmItemService;

    /**
     * @description: 查询报警定义
     * @param: data
     * @return: com.alibaba.fastjson.JSONArray
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 3:25 下午
     * @version: V1.0
     */
    @Deprecated
    public JSONArray queryAlarmConfig(JSONObject data) throws Exception {
        DmpResult<JSONArray> queryResult = alarmClient.queryAlarmConfig(getAlarmUrlParam(data), getRequestBody(data));
        JSONArray alarmConfigs = queryResult.getData();
        // 报警定义中的信息点转换为表号、功能号
        initAlarmConfigInfoCodes(alarmConfigs);
        return alarmConfigs;
    }

    /**
     * @description: 报警定义中的信息点转换为表号、功能号
     * @param: alarmConfigs
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/1 2:32 下午
     * @version: V1.0
     */
    public void initAlarmConfigInfoCodes(JSONArray alarmConfigs) throws Exception {
        if (alarmConfigs == null) {
            return;
        }
        for (Object alarmConfig : alarmConfigs) {
            JSONObject config = (JSONObject) alarmConfig;
            String classCode = config.getString("classCode");
            JSONObject condition = config.getJSONObject("condition");
            JSONArray infoCodeArray = condition.getJSONArray("infoCode");

            JSONArray infoCodes = new JSONArray();
            for (Object infoCodeObj : infoCodeArray) {
                JSONObject infoCode = new JSONObject();
                String infoCodeStr = (String) infoCodeObj;
                infoCode.put("infoCode", infoCodeStr);
                infoCode.put("meterId", getMeterId(infoCodeStr, classCode));
                infoCode.put("funcId", getFuncId(infoCodeStr, classCode));
                infoCodes.add(infoCode);
            }
            condition.put("infoCodes", infoCodes);
        }
    }

    /**
     * @description: 获取表号
     * @param: infoCode
     * @param: classCode
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/1 9:40 上午
     * @version: V1.0
     */
    public abstract String getMeterId(String infoCode, String classCode) throws Exception;

    /**
     * @description: 获取功能号
     * @param: infoCode
     * @param: classCode
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/1 9:40 上午
     * @version: V1.0
     */
    public abstract String getFuncId(String infoCode, String classCode) throws Exception;

    /**
     * @description: 获取报警名称
     * @param: objId
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 4:00 下午
     * @version: V1.0
     */
    public abstract String getAlarmName(String projectId,String objId,String category);

    /**
     * @description: 获取报警扩充字段
     * @param: objId
     * @return: JSONObject
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 4:00 下午
     * @version: V1.0
     */
    public abstract JSONObject getAlarmSupplement(AlarmUrlParam alarmUrlParam, JSONObject alarmRecord);

    /**
     * @description: 获取报警等级
     * @param: JSONObject 创建报警的参数
     * @return: 报警等级，默认为报警定义的报警等级
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 4:00 下午
     * @version: V1.0
     */
    public String getAlarmLevel(JSONObject data) {
        // 默认报警等级为低
        String defaultValue = "3";
        if (data != null && StringUtils.isNotBlank(data.getString("level"))) {
            return data.getString("level");
        }
        return defaultValue;
    }

    /**
     * @description: 获取报警备注
     * @param: objId
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 4:00 下午
     * @version: V1.0
     */
    public abstract String getAlarmRemark(AlarmUrlParam data, String objId, String itemCode);

    /**
     * @description: 创建报警id
     * @param: data
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 3:45 下午
     * @version: V1.0
     */
    public String createAlarm(JSONObject data) throws Exception {
        //报警分类（设备、空间、系统 Eq，Sp，Sy）
        String category = data.getString("category");
        String objId = data.getString("objId");
        String projectId = data.getString("projectId");
        String itemCode = data.getString("itemCode");

        data.put("classCode", getObjClassCode(getAlarmUrlParam(data), objId));
        data.put("category", getAlarmCategory(getAlarmUrlParam(data), itemCode));
        data.put("name", getAlarmName(projectId,objId,category));
        data.put("remark", getAlarmRemark(getAlarmUrlParam(data), objId, itemCode));
        data.put("supplement", getAlarmSupplement(getAlarmUrlParam(data), data));
        data.put("level", getAlarmLevel(data));

        DmpResult<JSONObject> alarmRecord = alarmClient.createAlarmRecord(getAlarmUrlParam(data), data);
        return alarmRecord.getData().getString("id");
    }

    /**
     * 获取报警类型所属的报警系统
     *
     * @param urlParam 数据中台通用param参数
     * @param itemCode 报警类型编码
     * @return 报警系统编码
     * @author lixing
     * @version V1.0 2021/11/4 10:15 下午
     */
    @SneakyThrows
    public String getAlarmCategory(AlarmUrlParam urlParam, String itemCode) {
        QueryAlarmItemDTO queryAlarmItemDTO = new QueryAlarmItemDTO();
        queryAlarmItemDTO.setCode(Lists.newArrayList(itemCode));
        queryAlarmItemDTO.setGroupCode(urlParam.getGroupCode());
        queryAlarmItemDTO.setProjectId(urlParam.getProjectId());
        queryAlarmItemDTO.setUserId(urlParam.getUserId());
        DmpResult<List<AlarmItem>> queryResult = alarmItemService.query(queryAlarmItemDTO);
        List<AlarmItem> data = DmpResultUtil.getData(queryResult);
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }

        return data.get(0).getCategory();
    }

    /**
     * 获取设备的设备类编码
     *
     * @param @param urlParam 数据中台通用param参数
     * @param objId 设备id
     * @return 设备编码
     * @author lixing
     * @version V1.0 2021/11/4 10:14 下午
     */
    protected abstract String getObjClassCode(AlarmUrlParam urlParam, String objId);

    /**
     * @description: 更新报警记录
     * @param: data
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 4:13 下午
     * @version: V1.0
     */
    public void updateAlarmRecord(JSONObject data) throws Exception {
        alarmClient.updateAlarmRecord(getAlarmUrlParam(data), data);
    }

    /**
     * @description: 报警定义增量变化时，查询发生变化的报警定义
     * @param: dmpMessage
     * @return: java.util.Map<java.lang.String, com.alibaba.fastjson.JSONArray>
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/1 12:06 下午
     * @version: V1.0
     */
    public Map<String, JSONArray> queryChangedAlarmConfigs(DmpMessage dmpMessage) throws Exception {
        String projectId = dmpMessage.getProjectId();
        JSONObject exts = dmpMessage.getExts();
        //删除报警定义
        JSONArray deletedConfigUniques = exts.getJSONArray("deletedConfigUniques");
        //新增报警定义
        JSONArray createdConfigUniques = exts.getJSONArray("createdConfigUniques");
        //修改报警定义
        JSONArray updatedConfigUniques = exts.getJSONArray("updatedConfigUniques");
        createdConfigUniques = CollectionUtils.isEmpty(createdConfigUniques) ? new JSONArray() : createdConfigUniques;
        deletedConfigUniques = CollectionUtils.isEmpty(deletedConfigUniques) ? new JSONArray() : deletedConfigUniques;
        updatedConfigUniques = CollectionUtils.isEmpty(updatedConfigUniques) ? new JSONArray() : updatedConfigUniques;
        createdConfigUniques.addAll(updatedConfigUniques);
        List<String> defineList = createdConfigUniques.stream().map(
                p -> JSONObject.parseObject(JSONObject.toJSONString(p))
        ).map(this::getAlarmConfigDefineId).collect(Collectors.toList());
        deletedConfigUniques = deletedConfigUniques.stream().map(
                p -> JSONObject.parseObject(JSONObject.toJSONString(p))
        ).filter(
                t -> !defineList.contains(getAlarmConfigDefineId(t))
        ).collect(Collectors.toCollection(JSONArray::new));

        JSONArray alarmConfigArr = new JSONArray();
        for (Object p : createdConfigUniques) {
            JSONObject jsonObject = JSONObject.parseObject(JSON.toJSONString(p));
            JSONObject data = new JSONObject();
            data.put("projectId", projectId);
            data.put("groupCode", dmpMessage.getGroupCode());
            data.put("itemCode", jsonObject.getString("itemCode"));
            data.put("objId", jsonObject.getString("objId"));
            data.put("userId", "system");
            DmpResult<JSONArray> alarmConfigQueryResult = alarmClient.queryAlarmConfig(getAlarmUrlParam(data), getRequestBody(data));
            JSONArray tmpAlarmConfigArr = alarmConfigQueryResult.getData();
            if (!CollectionUtils.isEmpty(tmpAlarmConfigArr)) {
                // 报警定义中的信息点转换为表号、功能号
                initAlarmConfigInfoCodes(tmpAlarmConfigArr);
                alarmConfigArr.addAll(tmpAlarmConfigArr);
            }
            log.info("新增和更新的报警定义-------------------:{}", tmpAlarmConfigArr);
        }
        Map<String, JSONArray> resultMap = new HashMap<>();
        resultMap.put("createdConfigUniques", alarmConfigArr);
        resultMap.put("deletedConfigUniques", deletedConfigUniques);
        return resultMap;
    }

    /**
     * @description: 获取报警定义唯一标识
     * @param: obj
     * @return: java.lang.String
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/1 11:54 上午
     * @version: V1.0
     */
    public String getAlarmConfigDefineId(JSONObject obj) {
        return obj.getString("itemCode") + "" + obj.getString("objId");
    }
}
