package com.persagy.apm.energyalarmstarter.alarmengine.service;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.AlarmUrlParam;
import com.persagy.apm.energyalarmstarter.alarmdata.mq.model.DmpMessage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;

/**
 * @description: 提供一些基础方法
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/12/16 10:10 上午
 * @version: V1.0
 **/
public class BaseService {
    /**
     * @description: 参数校验
     * @param: obj
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 3:14 下午
     * @version: V1.0
     */
    public void checkRequestParam(JSONObject obj) throws Exception {
        String projectId = obj.getString("projectId");
        String userId = obj.getString("userId");
        String groupCode = obj.getString("groupCode");
        if (StringUtils.isBlank(projectId)) {
            throw new Exception("projectId不能为空");
        } else if (StringUtils.isBlank(userId)) {
            throw new Exception("userId不能为空");
        } else if (StringUtils.isBlank(groupCode)) {
            throw new Exception("groupCode不能为空");
        }
    }

    /**
     * @description: 获取request中的param
     * @param: obj
     * @return: com.persagy.dmp.starter.alarm.feign.AlarmUrlParam
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 3:14 下午
     * @version: V1.0
     */
    public AlarmUrlParam getAlarmUrlParam(JSONObject obj) throws Exception {
        String projectId = obj.getString("projectId");
        // 如果projectId为空，设置为0，查询集团下所有项目的数据
        if (StringUtils.isEmpty(projectId)) {
            projectId = "0";
            obj.put("projectId", "0");
        } else {
            // 如果projectId为多个项目（用逗号分隔）, param中projectId传0
            String[] projectIds = projectId.split(",");
            if (projectIds.length > 1) {
                obj.put("projectId", "0");
            }
        }
        this.checkRequestParam(obj);

        return new AlarmUrlParam(
                obj.getString("userId"),
                obj.getString("groupCode"),
                projectId,
                obj.getString("appId")
        );
    }

    /**
     * 从数据中台消息中获取AlarmUrlParam
     *
     * @param msg 数据中台消息
     * @return AlarmUrlParam
     * @author lixing
     * @version V1.0 2021/11/19 10:15 下午
     */
    public AlarmUrlParam getAlarmUrlParam(DmpMessage msg) throws Exception {
        String projectId = StringUtils.isBlank(msg.getProjectId())? "0": msg.getProjectId();


        return new AlarmUrlParam(
                msg.getUserId(),
                msg.getGroupCode(),
                projectId,
                msg.getAppId()
        );
    }

    /**
     * @description: 获取request中的body
     * @param: obj
     * @return: com.alibaba.fastjson.JSONObject
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 3:14 下午
     * @version: V1.0
     */
    public JSONObject getRequestBody(JSONObject obj) throws Exception {
        JSONObject requestBody = new JSONObject();
        JSONObject criteria = (JSONObject) obj.clone();
        criteria.remove("appId");
        criteria.remove("userId");
        if (!CollectionUtils.isEmpty(criteria.getJSONArray("orders"))) {
            requestBody.put("orders", criteria.getJSONArray("orders"));
            criteria.remove("orders");
        }

        if (!CollectionUtils.isEmpty(criteria.getJSONArray("withColumns"))) {
            requestBody.put("withColumns", criteria.getJSONArray("withColumns"));
            criteria.remove("withColumns");
        }

        if (criteria.containsKey("onlyCount")) {
            requestBody.put("onlyCount", criteria.getBooleanValue("onlyCount"));
            criteria.remove("onlyCount");
        }

        if (criteria.containsKey("page") && criteria.containsKey("size")) {
            Integer page = criteria.getInteger("page");
            Integer size = criteria.getInteger("size");
            requestBody.put("page", page);
            requestBody.put("size", size);
            criteria.remove("page");
            criteria.remove("size");
        }
        /* 如果查询条件中包含projectId，进行如下处理 */
        String projectId = criteria.getString("projectId");
        // 如果projectId为空或是0，从查询条件中移除
        if (StringUtils.isEmpty(projectId) || "0".equals(projectId)) {
            criteria.remove("projectId");
        } else {
            // 将projectId转换为数组，适配projectId为多个项目（用逗号分隔）的情况
            String[] projectIds = projectId.split(",");
            criteria.put("projectId", projectIds);
        }

        /* 如果查询条件中包含groupCode, 且groupCode = 0, 从查询条件中移除groupCode */
        String groupCode = criteria.getString("groupCode");
        // 如果projectId为空或是0，从查询条件中移除
        if (StringUtils.isEmpty(groupCode) || "0".equals(groupCode)) {
            criteria.remove("groupCode");
        }

        requestBody.put("criteria", criteria);
        return requestBody;
    }
}
